/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.sg.financial.erpcredit.web;

import com.google.common.collect.Lists;
import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.web.BaseController;
import com.jeeplus.modules.sg.financial.erpcredit.entity.*;
import com.jeeplus.modules.sg.financial.erpcredit.service.ErpCreditService;
import com.jeeplus.modules.sg.financial.erpcredit.util.ErpInfo;
import com.jeeplus.modules.sg.financial.erpcredit.util.ImportUtil;
import com.jeeplus.modules.sg.financial.settlement.entity.MaintainData;
import com.jeeplus.modules.sg.financial.settlement.service.DataMaintenanceService;
import com.jeeplus.modules.test.one.dialog.entity.Leave1;
import com.jeeplus.modules.test.onetomany.dialog.entity.TestDataMain1;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.Row;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 数据维护
 */
@Controller
@RequestMapping(value = "${adminPath}/sg/financial/erpCredit")
public class ErpCreditController extends BaseController {

	@Autowired
	private ErpCreditService erpCreditService;

	@ModelAttribute
	public ErpCredit get(@RequestParam(required=false) String id) {
		ErpCredit entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = erpCreditService.get(id);
		}
		if (entity == null){
			entity = new ErpCredit();
		}
		return entity;
	}
	/**
	 * 表单列表页面
	 */
//	@RequiresPermissions("sg:financial:erpCredit:list")
	@RequestMapping(value = {"list", ""})
	public String list(ErpCredit erpCredit, Model model) {
//		model.addAttribute("leave1", erpCredit);
		model.addAttribute("erpCredit", erpCredit);
		return "modules/sg/financial/erp/erpCreditList";
	}

	/**
	 * 表单列表数据
	 */
	@ResponseBody
//	@RequiresPermissions("sg:financial:erpCredit:list")
	@RequestMapping(value = "data")
	public Map<String, Object> data(ErpCredit erpCredit, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<ErpCredit> page = erpCreditService.findPage(new Page<ErpCredit>(request, response), erpCredit);
		return getBootstrapData(page);
	}

	/**
	 * 查看，增加，编辑表单表单页面
	 */
//	@RequiresPermissions(value={"sg:financial:erpCredit:view","sg:financial:erpCredit:add","sg:financial:erpCredit:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(ErpCredit erpCredit, Model model) {
		model.addAttribute("erpCredit", erpCredit);
		return "modules/sg/financial/erp/erpCreditForm";
	}

	/**
	 * 保存表单
	 */
	@ResponseBody
//	@RequiresPermissions(value={"sg:financial:erpCredit:add","sg:financial:erpcredit:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public AjaxJson save(ErpCredit erpCredit, Model model) throws Exception{
		AjaxJson j = new AjaxJson();
		/**
		 * 后台hibernate-validation插件校验
		 */
		String errMsg = beanValidator(erpCredit);
		if (StringUtils.isNotBlank(errMsg)){
			j.setSuccess(false);
			j.setMsg(errMsg);
			return j;
		}
		//新增或编辑表单保存
		erpCreditService.save(erpCredit);//保存
		j.setSuccess(true);
		j.setMsg("保存表单成功");
		return j;
	}

	/**
	 * 删除表单
	 */
	@ResponseBody
//	@RequiresPermissions("sg:financial:erpCredit:del")
	@RequestMapping(value = "delete")
	public AjaxJson delete(ErpCredit erpCredit) {
		AjaxJson j = new AjaxJson();
		erpCreditService.delete(erpCredit);
		j.setMsg("删除表单成功");
		return j;
	}

	/**
	 * 批量请假表单
	 */
	@ResponseBody
//	@RequiresPermissions("sg:financial:erpCredit:del")
	@RequestMapping(value = "deleteAll")
	public AjaxJson deleteAll(String ids) {
		AjaxJson j = new AjaxJson();
		String idArray[] =ids.split(",");
		for(String id : idArray){
			erpCreditService.delete(erpCreditService.get(id));
		}
		j.setMsg("删除表单成功");
		return j;
	}

	/**
	 * 导出excel文件
	 */
	@ResponseBody
//	@RequiresPermissions("sg:financial:erpCredit:export")
	@RequestMapping(value = "export")
	public AjaxJson exportFile(ErpCredit erpCredit, HttpServletRequest request, HttpServletResponse response) {
		AjaxJson j = new AjaxJson();
		try {
			String fileName = "表单"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
			Page<ErpCredit> page = erpCreditService.findPage(new Page<ErpCredit>(request, response, -1), erpCredit);
			new ExportExcel("表单", ErpCredit.class).setDataList(page.getList()).write(response, fileName).dispose();
			j.setSuccess(true);
			j.setMsg("导出成功！");
			return j;
		} catch (Exception e) {
			j.setSuccess(false);
			j.setMsg("导出请单记录失败！失败信息："+e.getMessage());
		}
		return j;
	}
	@ResponseBody
	@RequestMapping(value = "detail")
	public ErpCredit detail(String id) {
		return erpCreditService.get(id);
	}
	/**
	 * 导入Excel数据
	 */
	@ResponseBody
//	@RequiresPermissions("sg:financial:erpCredit:import")
	@RequestMapping(value = "import")
	public AjaxJson importFile(@RequestParam("file")MultipartFile file, HttpServletResponse response, HttpServletRequest request){
		AjaxJson j = new AjaxJson();
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportUtil importUtil = new ImportUtil(file, 0, 0);
			List<ErpAccount> erpAccounts = ErpInfo.getErpAccount(importUtil);//获取表格数据
			List<ErpCredit> erpCredits = ErpInfo.getAllErpCreditList(erpAccounts);//获取单条数据
			for (ErpCredit erpCredit : erpCredits){
				try{
					erpCreditService.save(erpCredit);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条ERP账户记录。");
			}
			j.setMsg( "已成功导入 "+successNum+" 条ERP账户记录"+failureMsg);
		} catch (Exception e) {
			j.setSuccess(false);
			j.setMsg("导入ERP失败！失败信息："+e.getMessage());
		}
		return j;
	}
	/**
	 * 下载导入请假表单数据模板
	 */
	@ResponseBody
//	@RequiresPermissions("sg:financial:erpCredit:import")
	@RequestMapping(value = "import/template")
	public AjaxJson importFileTemplate(HttpServletResponse response) {
		AjaxJson j = new AjaxJson();
		try {
			String fileName = "请假表单数据导入模板.xlsx";
			List<ErpCredit> list = Lists.newArrayList();
			new ExportExcel("请假表单数据", ErpCredit.class, 1).setDataList(list).write(response, fileName).dispose();
			return null;
		} catch (Exception e) {
			j.setSuccess(false);
			j.setMsg( "导入模板下载失败！失败信息："+e.getMessage());
		}
		return j;
	}

}